package meerkat.voting;

import meerkat.protobuf.Voting;
import meerkat.protobuf.Voting.*;

/**
 * Created by hai on 23/02/16.
 */
public interface VotingBooth {

    public enum ErrorID {
        OUTPUT_DEVICE_OUT_OF_PAPER,
        OUTPUT_DEVICE_OUT_OF_INK,
        OUTPUT_DEVICE_NO_CONNECTION,
        OUTPUT_DEVICE_HARDWARE_NOT_FOUND,

        UI_HARDWARE_NOT_FOUND
    }


    // keeps track in which state we are
    // internal state will also include the number of the current question and the answers we got so far,
    // later it includes the details of the PlaintextBallot and later the Encryption and the secrets
    public enum State {
        INITIALIZED,

        VOTER_INTRODUCTION,
        VOTER_CHOOSE_CHANNEL,
        VOTER_VOTING,
        VOTER_ENCRYPTING,
        VOTER_CAST_OR_AUDIT,
        VOTER_CASTING,
        VOTER_AUDITING

    }

    public interface Callback {

        // callbacks and messages from BallotOutputDevice

        public void ballotCommitResult (long sessionId, boolean result);
        public void ballotCastResult (long sessionId, boolean result);
        public void ballotAuditResult (long sessionId, boolean result);
        public void ballotCancelResult (long sessionId, boolean result);
        // other error messages, such as Printer turned-off or out-of-paper
        public void outputDeviceReportProblem (ErrorID errorId);


        // callbacks and messages from UI

        public void introductionFinished (long sessionId);
        public void choiceOfChannel (long sessionId, int channelNumber);
        public void skipQuestion (long sessionId, int questionNumber);
        public void backwardsQuestion (long sessionId, int questionNumber);
        public void cancelVotingSession (long sessionId);
        public void answer (long sessionId, int questionNumber, String answer);
        public void cast (long sessionId);
        public void audit (long sessionId);
        // other error messages, such as hardware-not-detected
        public void uiReportProblem (ErrorID errorId);


        // callbacks from encryptor

        public void returnEncryption (long sessionId, EncryptedBallot encryptedBallot, BallotSecrets secrets);


        // callbacks from StorageManager

        public void adminHardwareKeyDetected (boolean isKeyInserted);
        public void loadElectionParams (ElectionParams electionParams);
    }

    // this function is synchronous
    public void start ();


    // messages from Admin Console (If there is such one)
    // this function is asynchronous
    public void shutDown();

}
